/**
  *  \file server/net.h
  *  \brief Networking Utilities
  */
#ifndef C2SERVER_SERVER_NET_H
#define C2SERVER_SERVER_NET_H

#include <sys/types.h>
#include <sys/socket.h>
#include "cpluslib/string.h"

/** Socket processor. A getaddrinfo wrapper. */
class SocketProcessor {
 public:
    virtual ~SocketProcessor();
    /** Check a socket.
        \param sock the socket
        \param addr address
        \param len  address length
        \retval true this socket was accepted, return it
        \retval false socket not accepted, try next one */
    virtual bool process(int sock, const sockaddr* addr, socklen_t len) = 0;

    /** Perform something with an address. Calls getaddrinfo to
        get all matching addresses, and then calls process()
        for each until one is accepted. Returns that socket.
        Returns -1 if none was accepted. */
    int withAddress(string_t host, string_t port);
};

/** Connect with a timeout. */
class TimedConnect : public SocketProcessor {
    const int secs;
 public:
    TimedConnect(int secs)
        : secs(secs)
        { }
    bool process(int sock, const sockaddr* addr, socklen_t len);
};

/** Connect. */
class Connect : public SocketProcessor {
 public:
    Connect()
        { }
    bool process(int sock, const sockaddr* addr, socklen_t len);
};

/** Bind to a socket. */
class Bind : public SocketProcessor {
 public:
    bool process(int sock, const sockaddr* addr, socklen_t len);
};

int timedRead(int sock, void* data, size_t size, int secs);
int timedWrite(int sock, const void* data, size_t size, int secs);
int timedFullWrite(int sock, const void* data, size_t size, int secs);
int timedWriteStr(int sock, const char* str, int secs);

#endif
